@@ -315,6 +315,7 @@ else { $passive = undef; }
 
 my $user_agent = LWP::UserAgent->new(env_proxy => 1);
 $user_agent->timeout($timeout);
+$user_agent->agent('Debian uscan ###VERSION###');
 
 if (defined $opt_watchfile) {
     die "Can't have directory arguments if using --watchfile" if @ARGV;
@@ -693,6 +694,12 @@ sub process_watchline ($$$$$$)
 	    (undef, $lastversion, $action) = split ' ', $line, 3;
 	}
 
+	# Check all's OK
+	if ($filepattern !~ /\(.*\)/) {
+	    warn "$progname warning: Filename pattern missing version delimiters ()\n  in $watchfile, skipping:\n  $line\n";
+	    return 1;
+	}
+
 	# Check validity of options
 	if ($base =~ /^ftp:/ and exists $options{'downloadurlmangle'}) {
 	    warn "$progname warning: downloadurlmangle option invalid for ftp sites,\n  ignoring in $watchfile:\n  $line\n";
@@ -823,17 +830,32 @@ sub process_watchline ($$$$$$)
 	# so we may have to look for <a href="filename"> type patterns
 	print STDERR "$progname debug: matching pattern $pattern\n" if $debug;
 	my (@files);
-	$content =~ s/\n/ \n/g; # make every filename have an extra
-	                        # space after it in a normal FTP listing
-	while ($content =~
-	           m&(?:<\s*a\s+[^>]*href\s*=\s*\"| )($pattern)(\"| )&gi) {
-	    my $file = $1;
-	    my $mangled_version = join(".", $file =~ m/^$pattern$/);
-	    foreach my $pat (@{$options{'uversionmangle'}}) {
-		eval "\$mangled_version =~ $pat;";
+
+	# We separate out HTMLised listings from standard listings, so
+	# that we can target our search correctly
+	if (/<\s*a\s+[^>]*href/i) {
+	    while ($content =~ 
+		m/(?:<\s*a\s+[^>]*href\s*=\s*\")($pattern)\"/gi) {
+		my $file = $1;
+		my $mangled_version = join(".", $file =~ m/^$pattern$/);
+		foreach my $pat (@{$options{'uversionmangle'}}) {
+		    eval "\$mangled_version =~ $pat;";
+		}
+		push @files, [$mangled_version, $file];
 	    }
-	    push @files, [$mangled_version, $file];
-	}
+	} else {
+	    # they all look like:
+	    # info info ... info filename [ -> linkname]
+	    while ($content =~ m/($filepattern)(\s+->\s+\S+)?$/mgi) {
+		my $file = $1;
+		my $mangled_version = join(".", $file =~ m/^$filepattern$/);
+		foreach my $pat (@{$options{'uversionmangle'}}) {
+		    eval "\$mangled_version =~ $pat;";
+		}
+		push @files, [$mangled_version, $file];
+	    }
+	}	    
+
 	if (@files) {
 	    if ($verbose) {
 		print "-- Found the following matching files:\n";
@@ -1015,7 +1037,11 @@ EOF
 	$request = HTTP::Request->new('GET', $upstream_url);
 	$response = $user_agent->request($request, "../$newfile_base");
 	if (! $response->is_success) {
-	    warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
+	    if (defined $pkg_dir) {
+		warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
+	    } else {
+		warn "$progname warning: Downloading\n $upstream_url failed:\n" . $response->status_line . "\n";
+	    }
 	    return 1;
 	}
     }
@@ -1032,7 +1058,11 @@ EOF
 	    else { delete $ENV{'FTP_PASSIVE'}; }
 	}
 	if (! $response->is_success) {
-	    warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
+	    if (defined $pkg_dir) {
+		warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
+	    } else {
+		warn "$progname warning: Downloading\n $upstream_url failed:\n" . $response->status_line . "\n";
+	    }
 	    return 1;
 	}
     }
@@ -1171,7 +1201,7 @@ sub recursive_regex_dir ($$$) {
 	    print STDERR "$progname debug: newest_dir => '$newest_dir'\n"
 		if $debug;
 	    if ($newest_dir ne '') {
-		$dir .= "$newest_dir/";
+		$dir .= "$newest_dir";
 	    }
 	    else {
 		return '';
@@ -1222,12 +1252,15 @@ sub newest_dir ($$$$$) {
 	    }
 	}
 	if (@hrefs) {
+	    @hrefs = Devscripts::Versort::versort(@hrefs);
 	    if ($debug) {
-		print "-- Found the following matching hrefs:\n";
+		print "-- Found the following matching hrefs (newest first):\n";
 		foreach my $href (@hrefs) { print "     $$href[1]\n"; }
 	    }
-	    @hrefs = Devscripts::Versort::versort(@hrefs);
-	    my ($newversion, $newdir) = @{$hrefs[0]};
+	    my $newdir = $hrefs[0][1];
+	    # just give the final directory component
+	    $newdir =~ s%/$%%;
+	    $newdir =~ s%^.*/%%;
 	    return $newdir;
 	} else {
 	    warn "$progname warning: In $watchfile,\n  no matching hrefs for pattern\n  $site$dir$pattern";
@@ -1265,18 +1298,29 @@ sub newest_dir ($$$$$) {
 	# so we may have to look for <a href="filename"> type patterns
 	print STDERR "$progname debug: matching pattern $pattern\n" if $debug;
 	my (@dirs);
-	$content =~ s/\n/ \n/g; # make every filename have an extra
-	                        # space after it in a normal FTP listing
-	while ($content =~
-	       m/(?:<\s*a\s+[^>]*href\s*=\s*\"| )($pattern)(\"| )/gi) {
-	    my $dir = $1;
-	    my $mangled_version = join(".", $dir =~ m/^$pattern$/);
-	    push @dirs, [$mangled_version, $dir];
-	}
+
+	# We separate out HTMLised listings from standard listings, so
+	# that we can target our search correctly
+	if (/<\s*a\s+[^>]*href/i) {
+	    while ($content =~ 
+		m/(?:<\s*a\s+[^>]*href\s*=\s*\")($pattern)\"/gi) {
+		my $dir = $1;
+		my $mangled_version = join(".", $dir =~ m/^$pattern$/);
+		push @dirs, [$mangled_version, $dir];
+	    }
+	} else {
+	    # they all look like:
+	    # info info ... info filename [ -> linkname]
+	    while ($content =~ m/($pattern)(\s+->\s+\S+)?$/mgi) {
+		my $dir = $1;
+		my $mangled_version = join(".", $dir =~ m/^$pattern$/);
+		push @dirs, [$mangled_version, $dir];
+	    }
+	}	    
 	if (@dirs) {
 	    if ($debug) {
-		print "-- Found the following matching dirs:\n";
-		foreach my $dir (@dirs) { print "     $$dir[1]\n"; }
+		print STDERR "-- Found the following matching dirs:\n";
+		foreach my $dir (@dirs) { print STDERR "     $$dir[1]\n"; }
 	    }
 	    @dirs = Devscripts::Versort::versort(@dirs);
 	    my ($newversion, $newdir) = @{$dirs[0]};
